@@ -8,6 +8,8 @@ from collections import UserList
 from .default_types import SimultanObject
 from .default_types import List as SimultanList
 from .default_types import ValueField, BuildInFace, BuildInVolume, BuildInZone
+from .geo_default_types import geometry_types
+from .geometry import GeometryModel
 
 from functools import lru_cache
 
@@ -75,7 +77,7 @@ class Template(object):
 
     def create_template_class(self, template_parser, template_classes):
         """
-        Create a template class of this template.
+        Creates a template class for this template.
         :param template_parser: TemplateParser instance
         :param template_classes: list of template classes to inherit from (e.g. default classes)
         :return: TemplateClass
@@ -86,22 +88,33 @@ class Template(object):
 
         self.template_parser = template_parser
 
+        # for inheritance:
+        # template classes must inherit from SimultanObject, UserList or ValueField
+
         if self.inherits_from is not None:
-            if self.inherits_from in self.template_parser.default_types.keys():
+            if self.inherits_from in self.template_parser.bases.keys():
                 base = template_classes[self.inherits_from]
-                if SimultanObject in base.__bases__:
-                    bases = (base, )
-                elif UserList in base.__bases__:
-                    bases = (base, )
-                elif ValueField in base.__bases__:
+
+                simultan_base_list = [SimultanObject, UserList, ValueField]
+
+                if any(x in simultan_base_list for x in base.__bases__):
                     bases = (base, )
                 else:
                     bases = (SimultanObject, ) + (base, )
+
+                # if SimultanObject in base.__bases__:
+                #     bases = (base, )
+                # elif UserList in base.__bases__:
+                #     bases = (base, )
+                # elif ValueField in base.__bases__:
+                #     bases = (base, )
+                # else:
+                #     bases = (SimultanObject, ) + (base, )
             else:
                 base = template_classes[self.inherits_from.template_name]
                 bases = (base, )
 
-            def new__init(self, *args, **kwargs):
+            def new_init(self, *args, **kwargs):
 
                 for i in range(self.__class__.__bases__.__len__()):
                     self.__class__.__bases__[i].__init__(self, *args, **kwargs)
@@ -110,11 +123,11 @@ class Template(object):
             bases = (SimultanObject, )
             # create the class from the template
 
-            def new__init(self, *args, **kwargs):
+            def new_init(self, *args, **kwargs):
 
                 self.__class__.__bases__[0].__init__(self, *args, **kwargs)
 
-        new_class_dict = {'__init__': new__init,
+        new_class_dict = {'__init__': new_init,
                           '_template_name': self.template_name,
                           '_template_id': self.template_id,
                           '_documentation': self.documentation,
@@ -161,14 +174,23 @@ class Template(object):
 
 class TemplateParser(object):
 
-    default_types = {'Liste': SimultanList,
-                     'List': SimultanList,
-                     'ValueField': ValueField,
-                     'Geometric Area': BuildInFace,
-                     'Geometrische_Flächen': BuildInFace,
-                     'Geometric Volume': BuildInVolume,
-                     'Geometrische_Volumina': BuildInVolume,
-                     'BuildInZone': BuildInZone}
+    bases = {'Liste': SimultanList,
+             'List': SimultanList,
+             'ValueField': ValueField,
+             'Geometric Area': BuildInFace,
+             'Geometrische_Flächen': BuildInFace,
+             'Geometric Volume': BuildInVolume,
+             'Geometrische_Volumina': BuildInVolume,
+             'BuildInZone': BuildInZone}
+
+    geo_bases = {'base': geometry_types.base,
+                 'layer': geometry_types.layer,
+                 'vertex': geometry_types.vertex,
+                 'edge': geometry_types.edge,
+                 'edge_loop': geometry_types.edge_loop,
+                 'face': geometry_types.face,
+                 'volume': geometry_types.volume,
+                 }
 
     _create_all = False
 
@@ -196,6 +218,8 @@ class TemplateParser(object):
 
         self.data_models = kwargs.get('data_models', {})
 
+        self._typed_geo_models = None
+
         template_classes[self.id] = self._template_classes
 
     @property
@@ -223,6 +247,18 @@ class TemplateParser(object):
             self.create_template_classes()
         return self._template_classes
 
+    @property
+    def typed_geo_models(self):
+        if self._typed_geo_models is None:
+            self._typed_geo_models = self.get_typed_geo_models()
+        return self._typed_geo_models
+
+    def get_model_by_file_id(self, id):
+        return self.current_data_model.models_dict[id]
+
+    def get_typed_model_by_file_id(self, id):
+        return self.typed_geo_models[id]
+
     def load_templates_from_file(self, filepath=None):
         """
         Load templates from file
@@ -239,7 +275,7 @@ class TemplateParser(object):
 
     def create_template_classes(self):
 
-        template_classes = {**self.default_types}
+        template_classes = {**self.bases}
 
         logger.info('\n\nCreating template-classes:\n-------------------------------------')
         logger.info(f'found {self.templates.__len__()} templates')
@@ -313,6 +349,17 @@ class TemplateParser(object):
         # new_instance._template_parser = self
         return new_instance
 
+    def get_typed_geo_models(self):
+        typed_models_dict = {}
+        for key, model in self.current_data_model.models_dict.items():
+            if model is None:
+                typed_models_dict[key] = None
+                continue
+            typed_models_dict[key] = GeometryModel(wrapped_obj=model,
+                                                   geo_types=self.geo_bases)
+            # models.append(GeometryModel(wrapped_obj=model))
+        return typed_models_dict
+
 
 def add_properties(prop, type):
     """
